var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { BigNumberString, DecimalString, HexString32, } from "@snickerdoodlelabs/objects";
import { ethers } from "ethers";
import { injectable } from "inversify";
import { fromString, toString } from "uint8arrays";
export const BigNumberBytes = 32;
let BigNumberUtils = class BigNumberUtils {
    constructor() { }
    //#region Math
    multiply(bn, number, decimals = 18) {
        const bnForSure = BigInt(bn);
        const multiplierBN = ethers.parseUnits(number.toString(), decimals);
        const oneBN = ethers.parseUnits("1", decimals);
        return (bnForSure * multiplierBN) / oneBN;
    }
    divide(bn, number, decimals = 18) {
        const bnForSure = BigInt(bn);
        const divisorBN = ethers.parseUnits(number.toString(), decimals);
        const oneBN = ethers.parseUnits("1", decimals);
        return (bnForSure * oneBN) / divisorBN;
    }
    //#endregion Math
    //#region DST
    /* Conversion from decimal string to big number and big number string */
    DSToBN(decimalString, decimals = 18) {
        return ethers.parseUnits(decimalString, decimals);
    }
    DSToBNS(decimalString, decimals) {
        return BigNumberString(ethers.parseUnits(decimalString, decimals).toString());
    }
    //#endregion DST
    //#region BN
    /* Conversion from big number to big number string and decimal string */
    BNtoBuffer(bigint) {
        const uint8Array = this.BNtoUint8Array(bigint);
        // Create a buffer from the Uint8Array in place
        //	 if we don't specify the byteOffset and byteLength we will copy data
        return Buffer.from(uint8Array, uint8Array.byteOffset, uint8Array.byteLength);
    }
    BNToBNS(bigint) {
        return BigNumberString(BigInt(bigint).toString());
    }
    BNToDS(bigNumber, decimals) {
        return DecimalString(ethers.formatUnits(bigNumber, decimals != undefined ? decimals : 18));
    }
    BNtoUint8Array(bigint) {
        let hexString = bigint.toString(16);
        // Ensure the hex string takes the required number of bytes
        //    each byte is represented by 2 hex characters
        //    so we need 32 bytes * 2 hex characters = 64 hex characters
        const requiredLength = BigNumberBytes * 2;
        while (hexString.length < requiredLength) {
            hexString = "0" + hexString;
        }
        return fromString(hexString, "base16");
    }
    //#endregion BN
    //#region BNS
    /* Conversion from big number string to big number and decimal string */
    BNStoBuffer(bigNumberString) {
        return this.BNtoBuffer(this.BNSToBN(bigNumberString));
    }
    BNSToBN(bigNumberString) {
        return BigInt(bigNumberString);
    }
    BNSToDS(bigNumberString, decimals) {
        return this.BNToDS(this.BNSToBN(bigNumberString), decimals);
    }
    BNStoHexString32(bigNumberString) {
        return HexString32(ethers.toBeHex(this.BNSToBN(bigNumberString), BigNumberBytes));
    }
    BNStoHexString32NoPrefix(bigNumberString) {
        return HexString32(this.BNStoHexString32(bigNumberString).substring(2));
    }
    BNStoUint8Array(bigNumberString) {
        return this.BNtoUint8Array(this.BNSToBN(bigNumberString));
    }
    //#endregion BNS
    //#region Bytes
    bufferToBN(buffer) {
        const hexString = HexString32(buffer.toString("hex"));
        return this.HexString32NoPrefixToBN(hexString);
    }
    bufferToBNS(buffer) {
        const hexString = HexString32(buffer.toString("hex"));
        return this.HexString32NoPrefixToBNS(hexString);
    }
    uint8ArrayToBN(uint8Array) {
        const hexString = HexString32(toString(uint8Array, "base16"));
        return this.HexString32NoPrefixToBN(hexString);
    }
    uint8ArrayToBNS(uint8Array) {
        const hexString = HexString32(toString(uint8Array, "base16"));
        return this.HexString32NoPrefixToBNS(hexString);
    }
    //#endregion Bytes
    //#region HexString32
    HexString32NoPrefixToBN(hexString) {
        return BigInt(`0x${hexString}`);
    }
    HexString32NoPrefixToBNS(hexString) {
        return BigNumberString(this.HexString32NoPrefixToBN(hexString).toString());
    }
    //#endregion HexString32
    //#region Validation
    /**
     * This method returns true if the bigNumberString is a
     * @param bigNumberString
     * @returns true if bigNumberString is a valid bigint
     */
    validateBNS(bigNumberString) {
        try {
            BigInt(bigNumberString);
            return true;
        }
        catch (e) {
            return false;
        }
    }
};
BigNumberUtils = __decorate([
    injectable(),
    __metadata("design:paramtypes", [])
], BigNumberUtils);
export { BigNumberUtils };
//# sourceMappingURL=BigNumberUtils.js.map